@@ -2,7 +2,6 @@
 Building Blocks of a Dynamic Visualization - :mod:`bpsci.core`
 ==============================================================
 The bpsci core is a collection of classes and methods used to create dynamic visualizations.
-
 """
 
 import numpy as np
@@ -11,10 +10,17 @@ from scipy.interpolate import interp1d
 import bpy
 import pandas as pd
 
+def arc_length_frac(x, y, z):
+    tmp = np.column_stack([np.diff(x) ** 2, np.diff(y) ** 2, np.diff(z) ** 2])
+    tmp2 = np.sqrt(np.sum(tmp, axis=1))
+    tmp3 = np.cumsum(tmp2)
+    tmp4 = tmp3 / tmp3[-1]
+    tmp4 = np.insert(tmp4, 0, 0)
+    return tmp4
+
 class init_anim:
     """
     Sets up the global animation information such speed up and global scale
-
     :param np.ndarray t: contains the time information that corresponds with the six degrees of freedom data 
     :param float speed_up: the 'real time speed up' or the ratio of the duration of the data to the duration of the animation 
     :param float scale: global physical scale factor of the animation, i.e., .1 will reduce everything to be 1/10th its original size
@@ -30,32 +36,32 @@ class init_anim:
         self.speed_up = speed_up
         """:type: `float`: the "real time speed up" or the ratio of the duration of the data to the duration of the animation"""
         
-        self.frames = np.linspace(1, int(t[-1]*self.frame_rate/speed_up+1), len(t)).astype(int)
-        """:type: `np.ndarray`: the frames that Blender will animate and have corresponding data for"""
-        
-        self.frame_duration = self.frames[-1]
+        self.frame_duration = int(t[-1]/speed_up*self.frame_rate)
         """:type: `int`: number of total frames in animation"""
 
+        frame_interper = interp1d(np.linspace(0, t[-1]*1.00001, len(t)), np.linspace(0, self.frame_duration, len(t)))
+
+        self.frames = frame_interper(t).astype(int)
+        """:type: `np.ndarray`: the frames that Blender will animate and have corresponding data for"""
+
         self.scale = scale
         """:type: `float`: global physical scale factor of the animation, i.e., .1 will reduce everything to be 1/10th its original size"""
         
         bpy.context.scene.frame_start = 1
         bpy.context.scene.frame_end = self.frame_duration+1
 
+
 class ref_frame:
     """
     Initializes a reference frame that other objects and animations can be the child of. 
-
     In blender, this takes the form of an Empty object.
 
-
     :param str name: the Blender object name of the reference frame
     :param bpy.data.object parent: the Blender object parent of this reference frame 
     :param anim: class object that was used to initialize the animation
     :type anim: :class:`bpsci.core.anim`
-    
-
     """
+
     def __init__(self, name, parent, anim):
 
         
@@ -91,14 +97,13 @@ class ref_frame:
         .. versionadded:: 0.2.30
 
         May be useful for offsetting the reference frame from the parent object (i.e. for center of mass of principal axes offset)
-
-
+        
         :param np.ndarray quat: an numpy array of one quaternion
         :param float x: the x position 
         :param float y: the y position 
         :param float z: the z position 
-
         """
+
         if not(quat is None):
             self.ob.rotation_quaternion[1] = quat[0]
             self.ob.rotation_quaternion[2] = quat[1]
@@ -112,18 +117,17 @@ class ref_frame:
     def dynamic_6DOF(self, quat, x_list, y_list, z_list):
         """
         Animates the reference frame over time.
-
+        
         .. warning::
             This is currently an internal method. This method will be exposed properly in future updates
         .. versionadded:: 0.2.30
-
+        
         May be useful for continually offsetting the reference frame from the parent object (i.e. for center of mass of principal axes offset for changing mass/distribution)
-
+        
         :param np.ndarray quat: an numpy array of quaternions over time
         :param np.ndarray x: the x position over time
         :param np.ndarray y: the y position over time
         :param np.ndarray z: the z position over time
-
         """
 
         frames = self.frames
@@ -155,17 +159,17 @@ class ref_frame:
 class dyn_obj:
     """
     The main building block of all dynamic visualizations 
-
-
+    
     :param bpy.data.object obj: the Blender object that will be animated
     :param np.ndarray pa: the offset of the principal axes specified by the :param: ` euler_type:
+    :param tuple cog: the center of gravity as defined from the origin of the 3D model
     :param str euler_type: the Euler angle order (i.e. 'xyz' for 1,2,3 or 'zxz' for 3,1,3)
     :param bpy.data.object parent: the Blender object parent of the original object 
     :param anim: class object that was used to initialize the animation
-    :type anim: :class:`bpsci.core.init_anim`
-        
+    :type anim: :class:`bpsci.core.init_anim`    
     """
-    def __init__(self, obj, pa, euler_type, parent, anim):
+
+    def __init__(self, obj, pa, cog, euler_type, parent, anim):
         
 
         self.parent = parent
@@ -199,16 +203,15 @@ class dyn_obj:
         obj.parent = self.body.ob
         obj.scale = (self.scale, self.scale, self.scale)
 
-        self.body.static_6DOF(-self.quat, None, None, None)
+        self.body.static_6DOF(-self.quat, cog[0], cog[1], cog[2])
 
         self.name = obj.name
         """:type: `str`: the Blender object name of the original object"""
 
     def apply_animation(self, x_list, y_list, z_list, quat_list):
-
         """
         Animates a :class:`~bpsci.core.dyn_obj` in the full six degrees of freedom
-
+        
         :param np.ndarray x_list: a numpy array of the x position over time
         :param np.ndarray y_list: a numpy array of the y position over time
         :param np.ndarray z_list: a numpy array of the z position over time
@@ -219,47 +222,57 @@ class dyn_obj:
         self.non_rot.dynamic_6DOF(None, x_list, y_list, z_list)
 
     def apply_streamline(self, staticity, int_x, int_y, int_z, thickness):
+        """
+        Gives a :class:`~bpsci.core.dyn_obj` a dynamic or static trail that shows its position
+        
+        :param str staticity: a string ['dynamic' or 'static'] that specifies whether the streamline is animated or static
+        :param np.ndarray x_list: a numpy array of the x position over time
+        :param np.ndarray y_list: a numpy array of the y position over time
+        :param np.ndarray z_list: a numpy array of the z position over time
+        :param np.ndarray quat_list: a numpy array of the quaternion over time. Can be passed None if rotation is ignored.
+        """
+
         name = self.name+'_streamline'
         frames = self.frames
-        
+
+        self.arc_length_frac = arc_length_frac(int_x, int_y, int_z)
+
         int_size = len(int_x)
         int_coords = np.zeros((int_size, 3))
         int_coords[:, 0] = int_x*self.scale
         int_coords[:, 1] = int_y*self.scale
         int_coords[:, 2] = int_z*self.scale
+
         int_curve = bpy.data.curves.new(name, type='CURVE')
         int_curve.dimensions = '3D'
         int_curve.resolution_u = 2
+
         # map coords to spline
         int_line = int_curve.splines.new('NURBS')
         int_line.points.add(len(int_coords))
         for i, coord in enumerate(int_coords):
             x,y,z = coord
             int_line.points[i].co = (x, y, z, 1)
+
         # create Object
         int_curveOB = bpy.data.objects.new(name, int_curve)
+
         # attach to scene and validate context
         bpy.context.collection.objects.link(int_curveOB)
 
         int_curveOB.data.bevel_depth = thickness
 
-        int_curveOB.data.use_fill_caps = True
-
-
-        int_curveOB.data.bevel_factor_start = 0
 
         if staticity == 'dynamic':
             for i in range(len(frames)):
-                cur_frame = frames[i]
-                
-                bpy.data.objects[name].data.bevel_factor_end = 1/(frames[-1]) * cur_frame
+                bpy.data.objects[name].data.bevel_factor_end = self.arc_length_frac[i]
                 bpy.data.objects[name].data.keyframe_insert(data_path='bevel_factor_end', frame=cur_frame)
+            
 
 class dyn_vec:
     """
     Initializes a dynamic vector`
-
-
+    
     :param bpy.data.object parent: the Blender object the vector is associated with
     :param str name: the Blender object name of the vector ('_vector' will be appended to this name)
     :param float scale_mag: the scaling factor of the vector's magnitude axis (purely aesthetic)
@@ -267,8 +280,8 @@ class dyn_vec:
     :param tuple[float] offset: offset of vector from the parent object (purely aesthetic)
     :param anim: class object that was used to initialize the animation
     :type anim: :class:`bpsci.core.anim`
-
     """
+
     def __init__(self, parent, name, scale_mag, scale_off, offset, anim):
 
         
@@ -292,6 +305,7 @@ class dyn_vec:
         """:type: `bpy.data.object`: the actual arrow asset"""
         
         self.parent_rf.ob.location = offset
+        self.offset = offset
         vec.parent = self.parent_rf.ob
         vec.name = self.name
         
@@ -305,11 +319,10 @@ class dyn_vec:
     def animate(self, x, y, z):
         """
         Animates a dynamic vector
-
+        
         :param np.ndarray x: a numpy array of the x component over time
         :param np.ndarray y: a numpy array of the y component over time
         :param np.ndarray z: a numpy array of the z component over time
-
         """
 
         frames = self.frames
@@ -346,7 +359,7 @@ class dyn_vec:
             
             self.parent_rf.ob.scale = (mag_arrow, 1, 1)
                 
-            self.point_rf.ob.location = (cur_x, cur_y, cur_z)
+            self.point_rf.ob.location = (cur_x + self.offset[0], cur_y+self.offset[1], cur_z+self.offset[2])
             
             self.point_rf.ob.keyframe_insert(data_path='location', frame=cur_frame)
             self.parent_rf.ob.keyframe_insert(data_path='scale', frame=cur_frame)
@@ -354,8 +367,7 @@ class dyn_vec:
 class anim_text:
     """
     Creates animated text
-
-
+    
     :param str name: the Blender object name of the text ('_text' will be appended to this name)
     :param bpy.data.object parent: the Blender object parent of this text 
     :param anim: class object that was used to initialize the animation
@@ -364,8 +376,8 @@ class anim_text:
     :param str label: a label to append to the text (if no label is desired, pass '')
     :param int fix_place: the number of decimal places to fix the text to
     :param bool if_str: whether or not the data is a numerical or string vector. Currently only numerical vectors are supported.
-
     """
+    
     def __init__(self, name, parent, anim, data, label, fix_place, if_str):
 
         
@@ -416,5 +428,4 @@ def curve_to_mesh(curve):
     context.view_layer.objects.active = new_obj
     
     bpy.data.objects.remove(curve, do_unlink=True)
-    new_obj.name = curve_name
-
+    new_obj.name = curve_name
\ No newline at end of file
